using System;
using System.Linq;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using EPiServer.PlugIn;
using EPiServer.Web.Hosting;
using logparserInterOp;

namespace EPiServerZA.Plugins.Admin
{
    [GuiPlugIn(DisplayName = "Download Statistics", Description = "Shows download statistics for downloaded files", Area = PlugInArea.AdminMenu, Url = "~/Plugins/Admin/DropboxStatistics.aspx")]
    public partial class DownloadStatistics : EPiServer.UI.SystemPageBase
    {
        protected override void OnPreInit(EventArgs e)
        {
            base.OnPreInit(e);
            this.MasterPageFile = ResolveUrlFromUI("MasterPages/EPiServerUI.master");
            this.SystemMessageContainer.Heading = "Download Statistics";
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (!IsPostBack)
            {
                rptVPPs.DataSource = from vpp in VirtualPathHandler.Instance.VirtualPathProviders where vpp.Key is VirtualPathUnifiedProvider select vpp.Key;
                rptVPPs.DataBind();

                base.SystemMessageContainer.Message = "Click 'Refresh' to load the statistics. Note that this can take up to a minute.";
            }
        }

        protected void Refresh(object sender, EventArgs e)
        {
            string likes = "";

            foreach (RepeaterItem item in rptVPPs.Items)
            {
                CheckBox chkVPP = (CheckBox)item.FindControl("chkVPP");

                if (chkVPP.Checked)
                {
                    if (likes != "") likes += " OR";
                    likes += @" cs-uri-stem LIKE '/" + chkVPP.Text + "/%'";
                }

                foreach (var control in item.Controls)
                {

                }
            }

            if (likes == "")
            {
                grdFiles.DataSource = null;
                grdFiles.DataBind();
                litCount.Text = "";

                base.SystemMessageContainer.Message = "You must select at least one provider to see stats";
                return;
            }

            var log = new LogQueryClassClass().Execute( @"SELECT date, time, cs-uri-stem from C:\inetpub\logs\LogFiles\W3SVC" + System.Web.Hosting.HostingEnvironment.ApplicationHost.GetSiteID() + @"\*.log " +
                                                        @"WHERE " + likes,
                                                        new COMIISW3CInputContextClassClass());

            // convert this non-standard collection into an enumerable

            var records = new List<ILogRecord>();

            while (!log.atEnd())
            {
                records.Add(log.getRecord());
                log.moveNext();
            }

            var results = from entry in records select new { Date = Convert.ToDateTime(entry.getValue("date")), Time = Convert.ToDateTime(entry.getValue("time")), VirtualPath = Convert.ToString(entry.getValue("cs-uri-stem")) };

            litCount.Text = "Total downloads: " + results.Count();

            if (radDownloadCount.Checked)
            {
                int rank = 1;

                grdFiles.DataSource = from result in results group result by result.VirtualPath into grouped orderby grouped.Count() descending select new { Rank = rank++, VirtualPath = grouped.Key, Downloads = grouped.Count() };
            }
            if (radDownloadDays.Checked)
            {
                int rank = 1;

                grdFiles.DataSource = from result in results group result by result.Date into grouped orderby grouped.Count() descending select new { Rank = rank++, Date = grouped.Key.ToShortDateString(), Downloads = grouped.Count() };
            }
            if (radDownloadHistorical.Checked)
            {
                grdFiles.DataSource = from date in Enumerable.Range(0, (new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day) - (from result in results select result.Date).Min()).Days + 1).Select(days => new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day).Subtract(new TimeSpan(days, 0, 0, 0))) join used in (from result in results group result by result.Date into grouped select new { Date = grouped.Key, Downloads = grouped.Count() }) on date equals used.Date into joined from item in joined.DefaultIfEmpty() orderby date descending select new { Date = date.ToShortDateString(), Downloads = (item == null) ? 0 : item.Downloads };
            }
            if (radDownloadList.Checked)
            {
                int download = results.Count();

                grdFiles.DataSource = from result in results orderby result.Date descending, result.Time descending select new { Download = download--, Date = result.Date.ToShortDateString(), Time = result.Time.ToShortTimeString(), VirtualPath = result.VirtualPath };
            }

            grdFiles.DataBind();
        }

        protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            for (int i = 0; i < e.Row.Cells.Count; i++)
            {
                e.Row.Cells[i].Attributes.Add("style", "white-space: nowrap;");
            }
        }
    }
}